#include "platno.h"
#include <qpainter.h>
#include <stdio.h>

double realRandom(){ return (((double)random())/RAND_MAX); }

Platno::Platno(QWidget *parent, const char *name) : QWidget(parent,name)
{ 
  FILE *f;
  f=fopen("krajinka.in","rt");
  fscanf(f,"%d %d %lf ",&MAPX,&MAPY,&fact);
  fclose(f);
  setPalette(QPalette(QColor(250,250,200))); computeWholeMap(); 
}

void Platno::Projekcia(double x, double y, double z, int *xx, int *yy)
{
  (*yy)=(int)(z+0.5*y);
  (*xx)=(int)(x+0.8660254*y);
}

void Platno::eraseMap() 
{
  int i,j;
  for (i=0;i<MAPX;i++) for (j=0;j<MAPY;j++) Map[i][j]=-1;
}

void Platno::computePartialMap(int xl, int yl, int xr, int yr)
{
  int xs,ys;
  double m,len;

//  fprintf(stderr,"[%d %d]-[%d %d] (rohy: %d %d %d %d)\n",xl,yl,xr,yr,Map[xl][yl],Map[xl][yr],Map[xr][yl],Map[xr][yr]);
  
//  if (xr<=xl+1) return; // a aby sme mali istotu:
//  if (yr<=yl+1) return;
  if ((xr<=xl+1) && (yr<=yl+1)) return;
  xs=(xl+xr)/2; ys=(yl+yr)/2;
  
  len=xr-xl+1;
  if (Map[xs][yl]==-1) {
    m=Map[xl][yl]+Map[xr][yl]; m/=2; m+=len*fact*realRandom(); m-=len*fact*0.5;
    if (m<0) m=0.1; if (m>255) m=255;
    Map[xs][yl]=(int)m;
  }
  if (Map[xs][yr]==-1) {
    m=Map[xl][yr]+Map[xr][yr]; m/=2; m+=len*fact*realRandom(); m-=len*fact*0.5;
    if (m<0) m=0.1; if (m>255) m=255;
    Map[xs][yr]=(int)m;
  }
  
  len=yr-yl+1;
  if (Map[xl][ys]==-1) {
    m=Map[xl][yl]+Map[xl][yr]; m/=2; m+=len*fact*realRandom(); m-=len*fact*0.5;
    if (m<0) m=0.1; if (m>255) m=255;
    Map[xl][ys]=(int)m;
  }
  if (Map[xr][ys]==-1) {
    m=Map[xr][yl]+Map[xr][yr]; m/=2; m+=len*fact*realRandom(); m-=len*fact*0.5;
    if (m<0) m=0.1; if (m>255) m=255;
    Map[xr][ys]=(int)m;
  }
  if (Map[xs][ys]==-1) {
    m=Map[xs][yl]+Map[xs][yr]; m/=2; m+=len*fact*realRandom(); m-=len*fact*0.5;
    if (m<0) m=0.1; if (m>255) m=255;
    Map[xs][ys]=(int)m;
  }
  
  computePartialMap(xl,yl,xs,ys); computePartialMap(xs,yl,xr,ys);
  computePartialMap(xl,ys,xs,yr); computePartialMap(xs,ys,xr,yr);
}

void Platno::computeWholeMap()
{
  int i,j;
  
 // fact=0.47;
  eraseMap();
//  fprintf(stderr,"Vymazal mapu\n");
/*
 * // OSTROV:
 * Map[0][0]=0;   Map[0][MAPY-1]=0;
 * Map[MAPX-1][0]=0;  Map[MAPX-1][MAPY-1]=0;
 * Map[(MAPX-1)/2][(MAPY-1)/2]=255; 
 */
  
  Map[0][0]=random()%256;   Map[0][MAPY-1]=random()%256;
  Map[MAPX-1][0]=random()%256;  Map[MAPX-1][MAPY-1]=random()%256;
  computePartialMap(0,0,MAPX-1,MAPY-1);
  repaint();
//  for (i=0;i<20;i++) for (j=0;j<20;j++) fprintf(stderr,"%d ",Map[i][j]);
}

void Platno::paintEvent(QPaintEvent *)
{
  int i,j,xx,yy1,yy2;
  int RGB[256][3];
  int horizont[3000];
  double x,y,z;
  QPainter p(this);
  
  for (i=  0;i< 20;i++) { RGB[i][0]=0;   RGB[i][1]=0;   RGB[i][2]=80; }
  for (i= 20;i< 64;i++) { RGB[i][0]=0;   RGB[i][1]=0;   RGB[i][2]=4*i;  }
  for (i= 64;i<128;i++) { RGB[i][0]=0;   RGB[i][1]=255-4*(i-64); RGB[i][2]=0;  }
//  for (i=128;i<192;i++) { RGB[i][0]=4*(i-128);   RGB[i][1]=2*(i-128); RGB[i][2]=0;  }
  for (i=128;i<160;i++) { RGB[i][0]=6*(i-128);   RGB[i][1]=3*(i-128); RGB[i][2]=0;  }
  for (i=160;i<192;i++) { RGB[i][0]=6*(191-i);   RGB[i][1]=3*(191-i); RGB[i][2]=0;  }
  for (i=192;i<240;i++) { RGB[i][0]=4*(i-176);   RGB[i][1]=4*(i-176); RGB[i][2]=4*(i-176);  }
  for (i=240;i<256;i++) { RGB[i][0]=255;   RGB[i][1]=255; RGB[i][2]=255;  }

//  for (i=0;i<=255;i++) fprintf(stderr,"color %d: r %d g %d b %d \n",i,RGB[i][0],RGB[i][1],RGB[i][2]);
  
  for (i=0;i<2*MAPX+width();i++) horizont[i]=height()+1;
  
  for (j=0;j<MAPY;j++) for (i=MAPX-1;i>=0;i--) {
    if ((Map[i][j]>=0) && (Map[i][j]<=255)) {
      x=i; y=j; z=0; Projekcia(x,y,z,&xx,&yy1);
      z=Map[i][j]; if (z<64) z=64; Projekcia(x,y,z,&xx,&yy2);
      yy1=(int)(0.9*height())-yy1; yy2=(int)(0.9*height())-yy2;
      xx+=(int)(0.1*width());
      if (yy2<horizont[xx]) {
        if (yy1>=horizont[xx]) yy1=horizont[xx]-1;
//        fprintf(stderr,"Setting color to : %d %d %d (Map[i][j]=%d)\n",RGB[Map[i][j]][0],RGB[Map[i][j]][1],RGB[Map[i][j]][2],Map[i][j]); 
        p.setPen(QColor(RGB[Map[i][j]][0],RGB[Map[i][j]][1],RGB[Map[i][j]][2])); 
        p.drawLine(xx,yy1,xx,yy2);
        horizont[xx]=yy2;
      }
    }
  }
  
}


QSizePolicy Platno::sizePolicy() const
{
    return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}

